import React, { useState, useRef, useEffect } from "react";
import { keyEvent } from "../util/keystrokeLogger";
import { KeyDetails } from "../util/types";

const SignUp = () => {
  const [alert, setAlert] = useState<string | null>(null);
  const [is2FAenabled, setis2FAenabled] = useState<boolean>(false);
  const [credentials, setCredentials] = useState({
    userName: "",
    password: "",
    confrimPassword: "",
    twoFAPassword: "",
  });

  const keydownArray = useRef<KeyDetails[][]>([]);
  const keyupArray = useRef<KeyDetails[][]>([]);
  const keyCount = useRef<number[]>([]);

  useEffect(() => {
    resetData();
  }, []);

  const resetData = () => {
    for (let i = 0; i < 3; i++) {
      keydownArray.current.push([]);
      keyupArray.current.push([]);
      keyCount.current.push(0);
    }
  };

  const onChange2FAconcent = () => setis2FAenabled(!is2FAenabled);
  const onChangeCredentials = (e: React.ChangeEvent<HTMLInputElement>) => {
    setCredentials({ ...credentials, [e.target.name]: e.target.value });
  };

  const onSumit = () => {
    if (credentials.password !== credentials.confrimPassword) {
      return setAlert("Passwords do not match");
    }

    const data = {
      username: credentials.userName,
      passwords: [
        credentials.password,
        credentials.confrimPassword,
        credentials.twoFAPassword,
      ],
      keydown: keydownArray.current,
      keyup: keyupArray.current,
    };

    console.log("DATA ", data);

    resetData();

    fetch("http://localhost:3001/user/signup", {
      method: "POST",
      mode: "cors",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify(data),
    }).then(async (res) => {
      const response = await res.json();

      if (response.success) {
        setAlert(`${response.username} signed up successfully`);
      } else {
        setAlert(response.msg);
      }
    });
  };

  const clearField = (index: number) => {
    if (index === 0) {
      setCredentials({ ...credentials, password: "" });
    } else if (index === 1) {
      setCredentials({ ...credentials, confrimPassword: "" });
    } else {
      setCredentials({ ...credentials, twoFAPassword: "" });
    }

    keydownArray.current[index] = [];
    keyupArray.current[index] = [];
    keyCount.current[index] = 0;
  };

  const onKeyDown = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    keydownArray.current[index].push(details);
    keyupArray.current[index].push({ ...details, time: null });
    keyCount.current[index]++;
  };

  const onKeyUp = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;
    if (!details.time) details.time = Date.now();

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    let reqdUpKeystroke = keyupArray.current[index].find(
      (element) => element.code === details?.code && !element.time
    );

    if (reqdUpKeystroke) reqdUpKeystroke.time = details.time;

    if (index === 2) {
      onSumit();
    }
  };

  const disableSubmit =
    credentials.userName === "" || credentials.password === "" || !is2FAenabled;

  const renderAlert = alert && (
    <div
      className="alert alert-warning alert-dismissible fade show mt-3"
      role="alert"
    >
      {alert}
      <button
        type="button"
        className="btn-close"
        data-bs-dismiss="alert"
        aria-label="Close"
        onClick={() => setAlert(null)}
      ></button>
    </div>
  );

  return (
    <div className="container">
      <div className=" row">
        <div className="col-lg"></div>
        <div className="col-lg">
          <div className="card card-body mt-5 pt-4 pb-4">
            <h5 className="card-title">SignUp</h5>
            <div className="mb-3">
              <label htmlFor="exampleFormControlInput1" className="form-label">
                User name
              </label>
              <input
                className="form-control"
                type="text"
                aria-label=".form-control-lg example"
                onChange={onChangeCredentials}
                name="userName"
              />
            </div>

            <div className="mb-3">
              <label htmlFor="exampleFormControlInput1" className="form-label">
                Password
              </label>
              <input
                className="form-control"
                type="password"
                aria-label=".form-control-lg example"
                name="password"
                onChange={onChangeCredentials}
                onKeyDown={onKeyDown}
                onKeyUp={onKeyUp}
                id="password-0"
                value={credentials.password}
              />
            </div>
            <div className="mb-3">
              <label htmlFor="exampleFormControlInput1" className="form-label">
                Confirm Password
              </label>
              <input
                className="form-control"
                type="password"
                aria-label=".form-control-lg example"
                name="confrimPassword"
                onChange={onChangeCredentials}
                value={credentials.confrimPassword}
                onKeyDown={onKeyDown}
                onKeyUp={onKeyUp}
                id="password-1"
              />
            </div>

            <div className="form-check mb-3">
              <input
                className="form-check-input"
                type="checkbox"
                name="2FA"
                id="flexCheckDefault"
                checked={is2FAenabled}
                onChange={onChange2FAconcent}
              />
              <label className="form-check-label" htmlFor="flexCheckDefault">
                Enable keystroke dynamics 2FA
              </label>
            </div>
            {is2FAenabled && (
              <div className="mb-3">
                <label
                  htmlFor="exampleFormControlInput1"
                  className="form-label"
                >
                  Enter your password again
                </label>
                <input
                  className="form-control"
                  type="password"
                  aria-label=".form-control-lg example"
                  name="twoFAPassword"
                  onChange={onChangeCredentials}
                  value={credentials.twoFAPassword}
                  onKeyDown={onKeyDown}
                  onKeyUp={onKeyUp}
                  id="password-2"
                />
              </div>
            )}
            <button
              type="button"
              className="btn btn-primary"
              disabled={disableSubmit}
              onClick={onSumit}
            >
              Sign up
            </button>
            {renderAlert}
          </div>
        </div>
        <div className="col-lg"></div>
      </div>
    </div>
  );
};

export default SignUp;
